@@ -36,7 +36,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 2003 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode2.c,v 1.3 2004/03/10 23:49:13 abe Exp $";
+static char *rcsid = "$Id: dnode2.c,v 1.4 2005/08/08 19:46:38 abe Exp $";
 #endif
 
 
@@ -73,7 +73,7 @@ readj2lino(ga, li)
 	    return(1);
 	li->dev = i.i_dev;
 	li->nlink = i.i_nlink;
-	li->number = i.i_number;
+	li->number = (INODETYPE)i.i_number;
 	li->size = i.i_size;
 	li->dev_def = li->nlink_def = li->number_def = li->size_def = 1;
 	return(0);
