@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: machine.h,v 1.40 2005/08/29 10:21:43 abe Exp $
+ * $Id: machine.h,v 1.42 2008/10/21 16:14:18 abe Exp $
  */
 
 
@@ -194,10 +194,19 @@ typedef	long long	aligned_offset_t  __attribute__ ((aligned (8)));
  * FSV_DEFAULT defines the default set of file structure values to list.
  * It defaults to zero (0), but may be made up of a combination of the
  * FSV_* symbols from lsof.h.
+ *
+ *   HASNOFSADDR  -- has no file structure address
+ *   HASNOFSFLAGS -- has no file structure flags
+ *   HASNOFSCOUNT -- has no file structure count
+ *   HASNOFSNADDR -- has no file structure node address
  */
 
 #define	HASFSTRUCT	1
 /* #define	FSV_DEFAULT	FSV_? | FSV_? | FSV_? */
+/* #define	HASNOFSADDR	1	has no file structure address */
+/* #define	HASNOFSFLAGS	1	has no file structure flags */
+/* #define	HASNOFSCOUNT	1	has no file structure count */
+/* #define	HASNOFSNADDR	1	has no file structure node address */
 
 
 /*
@@ -543,6 +552,15 @@ typedef	long long	aligned_offset_t  __attribute__ ((aligned (8)));
 /* #define	HASTCPTPIW	1 */
 
 
+/*
+ * HASTCPUDPSTATE is defined for dialects that have TCP and UDP state
+ * support -- i.e., for the "-stcp|udp:state" option and its associated
+ * speed improvements.
+ */
+
+#define	HASTCPUDPSTATE	1
+
+
 /*
  * HASTMPNODE is defined for those dialects that have tmpnodes.
  */
