@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.42 2009/03/25 19:23:06 abe Exp $
+ * $Id: dlsof.h,v 1.43 2010/01/18 19:03:01 abe Exp $
  */
 
 
@@ -66,8 +66,24 @@
 #   endif	/* FREEBSDV<6020 */
 #define	_KERNEL	1
 #  endif	/* FREEBSDV>=5000 */
+
+#  if	defined(HAS_VM_MEMATTR_T)
+/*
+ * The d_mmap2_t function typedef in <sys/conf.h> may need the definition
+ * of vm_memattr_t for a pointer, but that definition is only available
+ * under _KERNEL in <sys/types.h>.  Defining _KERNEL before including
+ * <sys/types.h> causes many compilation problems, so this expediency
+ * (hack) is used when the vm_memattr_t definition is needed.
+ */
+#define	vm_memattr_t	void
+#  endif	/* defined(HAS_VM_MEMATTR_T) */
+
 #include <sys/conf.h>
 
+#  if	defined(HAS_VM_MEMATTR_T)
+#undef	vm_memattr_t
+#  endif	/* defined(HAS_VM_MEMATTR_T) */
+
 #  if	defined(HAS_CONF_MINOR)
 #undef	minor
 #include "fbsd_minor.h"
@@ -186,9 +202,9 @@ struct vop_setextattr_args;
 #include <nfs/nfsproto.h>
 # endif	/* FREEBSDV<5000 */
 
-# if	defined(HASRPCV2H) || FREEBSDV>=4000
+# if	defined(HASRPCV2H)
 #include <nfs/rpcv2.h>
-# endif	/* defined(HASRPCV2H) || FREEBSDV>=4000 */
+# endif	/* defined(HASRPCV2H) */
 
 # if	FREEBSDV>=5000
 #include <nfsclient/nfs.h>
