@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dmnt.c,v 1.13 2006/03/27 23:29:03 abe Exp $";
+static char *rcsid = "$Id: dmnt.c,v 1.16 2009/03/25 19:23:06 abe Exp $";
 #endif
 
 
@@ -62,13 +62,21 @@ static char *mnt_names[] = INITMOUNTNAMES;
 
 #if	FREEBSDV>=5000 && defined(HAS_NO_SI_UDEV)
 /*
- * dev2udev() -- convert a kernel device number to a user device number
+ * Dev2Udev() -- convert a kernel device number to a user device number
  */
 
 dev_t
-dev2udev(c)
+Dev2Udev(c)
+
+# if	defined(HAS_CONF_MINOR) || defined(HAS_CDEV2PRIV)
+	KA_T c;
+# else	/* !defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV) */
 	struct cdev *c;
+# endif	/* defined(HAS_CONF_MINOR) || defined(HAS_CDEV2PRIV) */
+
 {
+
+# if	!defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV)
 	char *cp;
 	char *dn = (char *)NULL;
 	char *ln = (char *)NULL;
@@ -76,9 +84,27 @@ dev2udev(c)
 	int n, sr;
 	static u_int s;
 	struct stat sb;
-	static ss = 0;
+	static int ss = 0;
+# endif	/* !defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV) */
 
-# if	defined(HAS_SI_PRIV)
+# if	defined(HAS_CONF_MINOR) || defined(HAS_CDEV2PRIV)
+	KA_T ca;
+	struct cdev_priv cp;
+
+	if (!c)
+	    return(NODEV);
+
+#  if	defined(HAS_CDEV2PRIV)
+	ca = (KA_T)cdev2priv(c);
+#  else	/* !defined(HAS_CDEV2PRIV) */
+	ca = (KA_T)member2struct(cdev_priv, cdp_c, c);
+#  endif	/* defined(HAS_CDEV2PRIV) */
+
+	if (kread((KA_T)ca, (char *)&cp, sizeof(cp)))
+	    return(NODEV);
+	return((dev_t)cp.cdp_inode);
+# else	/* !defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV) */
+#  if	defined(HAS_SI_PRIV)
 /*
  * If the cdev structure has a private sub-structure, read it.
  */
@@ -86,15 +112,15 @@ dev2udev(c)
 
 	if (!c->si_priv || kread((KA_T)c->si_priv, (char *)&sp, sizeof(sp)))
 	    return(0);
-#endif	/* defined(HAS_SI_PRIV) */
+#  endif	/* defined(HAS_SI_PRIV) */
 
 	if (ss) {
 
-# if	defined(HAS_SI_PRIV)
+#  if	defined(HAS_SI_PRIV)
 	    return(sp.cdp_inode ^ s);
-# else	/* !defined(HAS_SI_PRIV) */
+#  else	/* !defined(HAS_SI_PRIV) */
 	    return(c->si_inode ^ s);
-#endif	/* defined(HAS_SI_PRIV) */
+#  endif	/* defined(HAS_SI_PRIV) */
 
 	}
 
@@ -108,11 +134,11 @@ dev2udev(c)
 	}
 	for (; n; n--, mb++) {
 
-#if	defined(MOUNT_NONE)
+# if	defined(MOUNT_NONE)
 	    if (mb->f_type == MOUNT_NONE || mb->f_type >= MOUNT_MAXTYPE)
-#else	/* !defined(MOUNT_NONE) */
+# else	/* !defined(MOUNT_NONE) */
 	    if (!mb->f_type)
-#endif	/* defined(MOUNT_NONE) */
+# endif	/* defined(MOUNT_NONE) */
 
 		continue;
 	/*
@@ -123,7 +149,7 @@ dev2udev(c)
 		(void) free((FREE_P *)dn);
 	    if (!(dn = mkstrcpy(mb->f_mntonname, (MALLOC_S *)NULL))) {
 
-dev2udev_no_space:
+Dev2Udev_no_space:
 
 		(void) fprintf(stderr, "%s: no space for mount at ", Pn);
 		safestrprt(mb->f_mntonname, stderr, 0);
@@ -153,16 +179,19 @@ dev2udev_no_space:
 	 */
 	    (void) free((FREE_P *)dn);
 	    if (!(dn = mkstrcpy(mb->f_mntfromname, (MALLOC_S *)NULL)))
-		goto dev2udev_no_space;
+		goto Dev2Udev_no_space;
 	    ln = Readlink(dn);
 	    if ((sr = statsafely(ln, &sb))) {
 
 	    /*
 	     * If the device stat(2) failed, see if the device name indicates
-	     * an NFS mount.  If it does, set the user device number seed to
-	     * zero.
+	     * an NFS mount, a cd9660 device, or a ZFS mount.  If any condition
+	     * is true, set the user device number seed to zero.
 	     */
-		if ((cp = strrchr(ln, ':')) && (*(cp + 1) == '/')) {
+		if (((cp = strrchr(ln, ':')) && (*(cp + 1) == '/'))
+		||  !strcasecmp(mb->f_fstypename, "cd9660")
+		||  !strcasecmp(mb->f_fstypename, "zfs")
+		) {
 		    ss = 1;
 		    s = (u_int)0;
 		}
@@ -193,16 +222,18 @@ dev2udev_no_space:
  */
 	if (ss) {
 
-# if	defined(HAS_SI_PRIV)
+#  if	defined(HAS_SI_PRIV)
 	    return(sp.cdp_inode ^ s);
-# else	/* !defined(HAS_SI_PRIV) */
+#  else	/* !defined(HAS_SI_PRIV) */
 	    return(c->si_inode ^ s);
-#endif	/* defined(HAS_SI_PRIV) */
+#  endif	/* defined(HAS_SI_PRIV) */
 
 	}
-
 	(void) fprintf(stderr, "%s: can't determine user device random seed.\n",	    Pn);
 	Exit(1);
+
+# endif	/* !defined(HAS_CONF_MINOR) */
+
 }
 #endif	/* FREEBSDV>=5000 && defined(HAS_NO_SI_UDEV) */
 
