@@ -32,17 +32,118 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.36 2006/03/27 23:29:03 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.40 2009/03/25 19:23:06 abe Exp $";
 #endif
 
 
 #include "lsof.h"
 
+#if	defined(HAS_LOCKF_ENTRY)
+#include "./lockf_owner.h"
+#endif	/* defined(HAS_LOCKF_ENTRY) */
+
+#if	defined(HAS_ZFS)
+#include "dzfs.h"
+#endif	/* defined(HAS_ZFS) */
+
 
 #if	defined(HASFDESCFS) && HASFDESCFS==1
 _PROTOTYPE(static int lkup_dev_tty,(dev_t *dr, INODETYPE *ir));
 #endif	/* defined(HASFDESCFS) && HASFDESCFS==1 */
 
+_PROTOTYPE(static void get_lock_state,(KA_T f));
+
+
+/*
+ * get_lock_state() -- get the lock state
+ */
+
+static void
+get_lock_state(f)
+	KA_T f;				/* inode's lock pointer */
+{
+	struct lockf lf;		/* lockf structure */
+	int lt;				/* lock type */
+
+#if	defined(HAS_LOCKF_ENTRY)
+	struct lockf_entry le;		/* lock_entry structure */
+	KA_T lef, lep;			/* lock_entry pointers */
+	struct lock_owner lo;		/* lock owner structure */
+
+	if (!f || kread(f, (char *)&lf, sizeof(lf)))
+	    return;
+	if (!(lef = (KA_T)lf.ls_active.lh_first))
+	    return;
+	lep = lef;
+	do {
+	   if (kread(lep, (char *)&le, sizeof(le)))
+		return;
+	    if (!le.lf_owner
+	    ||  kread((KA_T)le.lf_owner, (char *)&lo, sizeof(lo)))
+		continue;
+	    if (lo.lo_pid == (pid_t)Lp->pid) {
+		if (le.lf_start == (off_t)0
+		&&  le.lf_end == 0x7fffffffffffffffLL)
+		    lt = 1;
+		else
+		    lt = 0;
+		if (le.lf_type == F_RDLCK)
+		    Lf->lock = lt ? 'R' : 'r';
+		else if (le.lf_type == F_WRLCK)
+		    Lf->lock = lt ? 'W' : 'w';
+		else if (le.lf_type == (F_RDLCK | F_WRLCK))
+		    Lf->lock = 'u';
+		return;
+	    }
+	} while ((lep = (KA_T)le.lf_link.le_next) && (lep != lef));
+#else	/* !defined(HAS_LOCKF_ENTRY) */
+
+	unsigned char l;		/* lock status */
+	KA_T lfp;			/* lockf structure pointer */
+
+	if ((lfp = f)) {
+
+	/*
+	 * Determine the lock state.
+	 */
+	    do {
+		if (kread(lfp, (char *)&lf, sizeof(lf)))
+		    break;
+		l = 0;
+		switch (lf.lf_flags & (F_FLOCK|F_POSIX)) {
+		case F_FLOCK:
+		    if (Cfp && (struct file *)lf.lf_id == Cfp)
+			l = 1;
+		    break;
+		case F_POSIX:
+
+# if	defined(P_ADDR)
+		    if ((KA_T)lf.lf_id == Kpa)
+			l = 1;
+# endif	/* defined(P_ADDR) */
+
+		    break;
+		}
+		if (!l)
+		    continue;
+		if (lf.lf_start == (off_t)0
+		&&  lf.lf_end == 0xffffffffffffffffLL)
+		    lt = 1;
+		else
+		    lt = 0;
+		if (lf.lf_type == F_RDLCK)
+		    Lf->lock = lt ? 'R' : 'r';
+		else if (lf.lf_type == F_WRLCK)
+		    Lf->lock = lt ? 'W' : 'w';
+		else if (lf.lf_type == (F_RDLCK | F_WRLCK))
+		    Lf->lock = 'u';
+		break;
+	    } while ((lfp = (KA_T)lf.lf_next) && (lfp != f));
+   	}
+#endif	/* defined(HAS_LOCKF_ENTRY) */
+
+}
+
 
 #if	FREEBSDV>=2000
 # if	defined(HASPROCFS)
@@ -158,12 +259,11 @@ process_node(va)
 {
 	dev_t dev, rdev;
 	unsigned char devs;
-	unsigned char lt;
 	unsigned char rdevs;
 	char dev_ch[32], *ep;
 	struct inode *i;
-	struct lockf lf, *lff, *lfp;
 	struct nfsnode *n;
+	size_t sz;
 	char *ty;
 	enum vtype type;
 	struct vnode *v, vb;
@@ -172,12 +272,13 @@ process_node(va)
 #if	FREEBSDV>=2000
 	struct inode ib;
 	struct nfsnode nb;
-	size_t sz;
 # if	FREEBSDV>=4000
 #  if	FREEBSDV<5000
 	struct specinfo si;
 #  else	/* FREEBSDV>=5000 */
+#   if	!defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV)
 	struct cdev si;
+#   endif	/* !defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV) */
 #  endif	/* FREEBSDV<5000 */
 # endif	/* FREEBSDV>=4000 */
 #endif	/* FREEBSDV>=2000 */
@@ -220,14 +321,17 @@ process_node(va)
 	struct ufs2_dinode d2;
 # endif	/* !defined(HAS_UFS1_2) */
 
+# if	!defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV)
 	struct cdev cd;
+# endif	/* !defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV) */
+
 	int cds;
 	struct devfs_dirent de;
 	struct devfs_dirent *d;
 	char vtbuf[32];
 	char *vtbp;
 	enum vtagtype { VT_DEVFS, VT_FDESC, VT_ISOFS, VT_PSEUDOFS, VT_NFS,
-			VT_NULL, VT_UFS, VT_UNKNOWN
+			VT_NULL, VT_UFS, VT_ZFS, VT_UNKNOWN
 		      };
 #endif	/* FREEBSDV>=5000 */
 
@@ -251,10 +355,18 @@ process_node(va)
 # endif	/* FREEBSDV>=2000 */
 #endif	/* defined(HASPROCFS) */
 
-# if	defined(HASPSEUDOFS) 
+#if	defined(HASPSEUDOFS) 
 	struct pfs_node pn;
 	struct pfs_node *pnp;
-# endif	/* defined(HASPSEUDOFS) */
+#endif	/* defined(HASPSEUDOFS) */
+
+#if	defined(HAS_ZFS)
+	zfs_info_t *z = (zfs_info_t *)NULL;
+	zfs_info_t zi;
+	char *zm = (char *)NULL;
+#else	/* !defined(HAS_ZFS) */
+	static unsigned char zw = 0;
+#endif	/* HAS_VFS */
 
 	enum vtagtype vtag;			/* placed here to use the
 						 * artificial vtagtype
@@ -303,9 +415,14 @@ process_overlaid_node:
 	p = (struct pfsnode *)NULL;
 #endif	/* defined(HASPROCFS) */
 
-# if	defined(HASPSEUDOFS) 
+#if	defined(HASPSEUDOFS) 
 	pnp = (struct pfs_node *)NULL;
-# endif	/* defined(HASPSEUDOFS) */
+#endif	/* defined(HASPSEUDOFS) */
+
+#if	defined(HAS_ZFS)
+	z = (zfs_info_t *)NULL;
+	zm = (char *)NULL;
+#endif	/* defined(HAS_ZFS) */
 
 /*
  * Read the vnode.
@@ -386,7 +503,11 @@ process_overlaid_node:
  */
 	if (((v->v_type == VCHR) || (v->v_type == VBLK))
 	&&  v->v_rdev
+
+# if	!defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV)
 	&&  !kread((KA_T)v->v_rdev, (char *)&cd, sizeof(cd))
+# endif	/* !defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV) */
+
 	) {
 	    cds = 1;
 	}
@@ -407,7 +528,22 @@ process_overlaid_node:
 	    vtbp = vtbuf;
 	    if (!strcmp(vtbuf, "ufs"))
 		vtag = VT_UFS;
-	    else if (!strcmp(vtbuf, "devfs"))
+	    else if (!strcmp(vtbuf, "zfs")) {
+
+#if	!defined(HAS_ZFS)
+		if (!Fwarn && !zw) {
+		    (void) fprintf(stderr,
+			"%s: WARNING: no ZFS support has been defined.\n",
+			Pn);
+		    (void) fprintf(stderr,
+			"      See 00FAQ for more information.\n");
+		    zw = 1;
+		}
+#else	/* defined(HAS_ZFS) */
+		vtag = VT_ZFS;
+#endif	/* !defined(HAS_ZFS) */
+
+	    } else if (!strcmp(vtbuf, "devfs"))
 		vtag = VT_DEVFS;
 	    else if (!strcmp(vtbuf, "nfs"))
 		vtag = VT_NFS;
@@ -631,47 +767,38 @@ process_overlaid_node:
 # endif	/* defined(HAS_UFS1_2) */
 #endif	/* FREEBSDV<2000 */
 
-	    if ((lff = i->i_lockf)) {
+#if	defined(HAS_V_LOCKF)
+	    if (v->v_lockf)
+		(void) get_lock_state((KA_T)v->v_lockf);
+#else	/* !defined(HAS_V_LOCKF) */
+	    if (i->i_lockf)
+		(void) get_lock_state((KA_T)i->i_lockf);
+#endif	/* defined(HAS_V_LOCKF) */
 
-	    /*
-	     * Determine the lock state.
-	     */
-		lfp = lff;
-		do {
-		    if (kread((KA_T)lfp, (char *)&lf, sizeof(lf)))
-			break;
-		    lt = 0;
-		    switch (lf.lf_flags & (F_FLOCK|F_POSIX)) {
-		    case F_FLOCK:
-			if (Cfp && (struct file *)lf.lf_id == Cfp)
-			    lt = 1;
-			break;
-		    case F_POSIX:
-
-#if	defined(P_ADDR)
-			if ((KA_T)lf.lf_id == Kpa)
-			    lt = 1;
-#endif	/* defined(P_ADDR) */
-
-			break;
-		    }
-		    if (!lt)
-			continue;
-		    if (lf.lf_start == (off_t)0
-		    &&  lf.lf_end == 0xffffffffffffffffLL)
-			lt = 1;
-		    else
-			lt = 0;
-		    if (lf.lf_type == F_RDLCK)
-			Lf->lock = lt ? 'R' : 'r';
-		    else if (lf.lf_type == F_WRLCK)
-			Lf->lock = lt ? 'W' : 'w';
-		    else if (lf.lf_type == (F_RDLCK | F_WRLCK))
-			Lf->lock = 'u';
-		    break;
-		} while ((lfp = lf.lf_next) && lfp != lff);
-	    }
 	    break;
+
+#if	defined(HAS_ZFS)
+	case VT_ZFS:
+	    if (!v->v_data
+	    ||  (zm = readzfsnode((KA_T)v->v_data, &zi))) {
+		(void) snpf(Namech, Namechl, "%s: %s", zm,
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
+		enter_nm(Namech);
+		return;
+	    }
+	    z = &zi;
+
+#if	defined(HAS_V_LOCKF)
+	    if (v->v_lockf)
+		(void) get_lock_state((KA_T)v->v_lockf);
+#else	/* !defined(HAS_V_LOCKF) */
+	    if (z->lockf)
+		(void) get_lock_state((KA_T)z->lockf);
+#endif	/* defined(HAS_V_LOCKF) */
+
+	    break;
+#endif	/* defined(HAS_ZFS) */
+
 	default:
 	    if (v->v_type == VBAD || v->v_type == VNON)
 		break;
@@ -700,10 +827,19 @@ process_overlaid_node:
 
 #if	FREEBSDV>=4000
 	    if (i->i_dev
-	    &&  !kread((KA_T)i->i_dev, (char *)&si, sizeof(si))) {
+
+# if	!defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV)
+	    &&  !kread((KA_T)i->i_dev, (char *)&si, sizeof(si))
+# endif	/* !defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV) */
+
+	    ) {
 
 # if	defined(HAS_NO_SI_UDEV)
-		dev = dev2udev(&si);
+#  if	defined(HAS_CONF_MINOR) || defined(HAS_CDEV2PRIV)
+		dev = Dev2Udev((KA_T)i->i_dev);
+#  else	/* !defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV) */
+		dev = Dev2Udev(&si);
+#  endif	/* defined(HAS_CONF_MINOR) || defined(HAS_CDEV2PRIV) */
 # else	/* !defined(HAS_NO_SI_UDEV) */
 		dev = si.si_udev;
 # endif	/* defined(HAS_NO_SI_UDEV) */
@@ -731,7 +867,11 @@ process_overlaid_node:
 		if (cds) {
 
 # if	defined(HAS_NO_SI_UDEV)
-		    rdev = dev2udev(&cd);
+#  if	defined(HAS_CONF_MINOR) || defined(HAS_CDEV2PRIV)
+		    rdev = Dev2Udev((KA_T)v->v_rdev);
+#  else	/* !defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV) */
+		    rdev = Dev2Udev(&cd);
+#  endif	/* defined(HAS_CONF_MINOR) || defined(HAS_CDEV2PRIV) */
 # else	/* !defined(HAS_NO_SI_UDEV) */
 		    rdev = cd.si_udev;
 # endif	/* defined(HAS_NO_SI_UDEV) */
@@ -746,6 +886,25 @@ process_overlaid_node:
 	    }
 	}
 
+#if	defined(HAS_ZFS)
+	else if (z) {
+
+	/*
+	 * Record information returned by readzfsnode().
+	 */
+	    if (vfs) {
+		dev = vfs->fsid.val[0];
+		devs = 1;
+	    }
+	    if ((type == VCHR) || (type == VBLK)) {
+		if (z->rdev_def) {
+		    rdev = z->rdev;
+		    rdevs = 1;
+		}
+	    }
+	}
+#endif	/* defined(HAS_ZFS) */
+
 #if	defined(HASFDESCFS) && (defined(HASFDLINK) || HASFDESCFS==1)
 	else if (f) {
 
@@ -805,7 +964,11 @@ process_overlaid_node:
 		if (cds) {
 
 # if	defined(HAS_NO_SI_UDEV)
-		    rdev = dev2udev(&cd);
+#  if	defined(HAS_CONF_MINOR) || defined(HAS_CDEV2PRIV)
+		    rdev = Dev2Udev((KA_T)v->v_rdev);
+#  else	/* !defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV) */
+		    rdev = Dev2Udev(&cd);
+#  endif	/* defined(HAS_CONF_MINOR) || defined(HAS_CDEV2PRIV) */
 # else	/* !defined(HAS_NO_SI_UDEV) */
 		    rdev = cd.si_udev;
 # endif	/* defined(HAS_NO_SI_UDEV) */
@@ -831,7 +994,18 @@ process_overlaid_node:
 	if (i) {
 	    Lf->inode = (INODETYPE)i->i_number;
 	    Lf->inp_ty = 1;
-	} else if (n) {
+	}
+
+#if	defined(HAS_ZFS)
+	else if (z) {
+	    if (z->ino_def) {
+		Lf->inode = z->ino;
+		Lf->inp_ty = 1;
+	    }
+	}
+#endif	/* defined(HAS_ZFS) */
+
+	else if (n) {
 	    Lf->inode = (INODETYPE)n->n_vattr.va_fileid;
 	    Lf->inp_ty = 1;
 	}
@@ -945,6 +1119,16 @@ process_overlaid_node:
 			Lf->sz_def = 1;
 		    }
 
+
+#if     defined(HAS_ZFS)
+		    else if (z) {
+			if (z->sz_def) {
+			    Lf->sz = z->sz;
+			    Lf->sz_def = 1;
+			}
+		    }
+#endif  /* defined(HAS_ZFS) */
+
 #if	FREEBSDV<5000
 		    else if (m) {
 			Lf->sz = (SZOFFTYPE)m->mfs_size;
@@ -988,6 +1172,15 @@ process_overlaid_node:
 		    Lf->nlink_def = 1;
 		}
 
+#if	defined(HAS_ZFS)
+		else if (z) {
+		    if (z->nl_def) {
+			Lf->nlink = z->nl;
+			Lf->nlink_def = 1;
+		    }
+		}
+#endif	/* defined(HAS_ZFS) */
+
 #if	defined(HAS9660FS)
 		else if (iso_stat) {
 		    Lf->nlink = iso_links;
@@ -1265,7 +1458,8 @@ process_pipe(pa)
 	    return;
 	}
 	(void) snpf(Lf->type, sizeof(Lf->type), "PIPE");
-	(void) snpf(dev_ch, sizeof(dev_ch), "%#x", pa);
+	(void) snpf(dev_ch, sizeof(dev_ch), "%s",
+	    print_kptr(pa, (char *)NULL, 0));
 	enter_dev_ch(dev_ch);
 	if (Foffset)
 	    Lf->off_def = 1;
