@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.26 2005/08/08 19:49:23 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.28 2009/03/25 19:23:06 abe Exp $";
 #endif
 
 
@@ -47,9 +47,100 @@ static char *rcsid = "$Id: dsock.c,v 1.26 2005/08/08 19:49:23 abe Exp $";
  */
 
 #define IPv6_2_IPv4(v6)	(((uint8_t *)((struct in6_addr *)v6)->s6_addr)+12)
+
+# if	defined(HAS_NO_6PORT)
+/*
+ * If the in_pcb structure no longer has the KAME accommodations of
+ * in6p_[fl]port, redefine them to inp_[fl]port.
+ */
+
+#define	in6p_fport	inp_fport
+#define	in6p_lport	inp_lport
+# endif	/* defined(HAS_NO_6PORT) */
+
+# if	defined(HAS_NO_6PPCB)
+/*
+ * If the in_pcb structure no longer has the KAME accommodation of in6p_pcb,
+ * redefine it to inp_ppcb.
+ */
+
+#define	in6p_ppcb	inp_ppcb
+# endif	/* defined(HAS_NO_6PPCB) */
 #endif	/* defined(HASIPv6) */
 
 
+/*
+ * Local function prototypes
+ */
+
+_PROTOTYPE(static int ckstate,(KA_T ta, struct tcpcb *t, int fam));
+
+
+/*
+ * ckstate() -- read TCP control block and check TCP state for inclusion
+ *		or exclusion
+ * return: -1 == no TCP CB available
+ *	    0 == TCP DB available; continue processing file
+ *	    1 == stop processing file
+ */
+
+static int
+ckstate(ta, t, fam)
+	KA_T ta;			/* TCP control block address */
+	struct tcpcb *t;		/* TCP control block receptor */
+	int fam;			/* protocol family */
+{
+	int tsnx;
+/*
+ * Read TCP control block.
+ */
+	if (kread(ta, (char *)t, sizeof(struct tcpcb)))
+	    return(-1);
+	if (TcpStXn || TcpStIn) {
+
+	/*
+	 * If there are TCP state inclusions or exclusions, check them.
+	 */
+	    tsnx = (int)t->t_state + TcpStOff;
+	    if (TcpStXn) {
+		if (TcpStX[tsnx]) {
+		    Lf->sf &= ~SELNET;
+		    Lf->sf |= SELEXCLF;
+		    return(1);
+		}
+	    }
+	    if (TcpStIn) {
+		if (TcpStI[tsnx]) {
+		    TcpStI[tsnx] = 2;
+		    Lf->sf |= SELNET;
+		} else {
+		    Lf->sf &= ~SELNET;
+		    Lf->sf |= SELEXCLF;
+		    return(1);
+		}
+	    }
+	}
+	if (!(Lf->sf & SELNET) && !TcpStIn) {
+
+	/*
+	 * See if this TCP file should be selected.
+	 */
+	    if (Fnet) {
+		if (!FnetTy
+		||  (FnetTy == 4) && (fam == AF_INET)
+
+#if	defined(HASIPv6)
+		||  (FnetTy == 6) && (fam == AF_INET6)
+#endif 	/* defined(HASIPv6) */
+
+		) {
+		    Lf->sf |= SELNET;
+		}
+	    }
+	}
+	return(0);
+}
+
 
 /*
  * process_socket() - process socket
@@ -68,7 +159,7 @@ process_socket(sa)
 	struct protosw p;
 	struct socket s;
 	struct tcpcb t;
-	KA_T ta = (KA_T)NULL;
+	int ts = -1;
 	struct unpcb uc, unp;
 	struct sockaddr_un *ua = NULL;
 	struct sockaddr_un un;
@@ -176,9 +267,11 @@ process_socket(sa)
 #if	defined(HASIPv6)
 		||  ((FnetTy == 6) && (fam == AF_INET6))
 #endif	/* defined(HASIPv6) */
-		)
 
-		    Lf->sf |= SELNET;
+		) {
+		    if (!TcpStIn && !UdpStIn)
+			Lf->sf |= SELNET;
+		}
 	    }
 	    printiproto(p.pr_protocol);
 
@@ -205,11 +298,15 @@ process_socket(sa)
 	    /*
 	     * Save IPv6 address information.
 	     */
+		if (p.pr_protocol == IPPROTO_TCP) {
+		    if (in6p.in6p_ppcb) {
+			if ((ts = ckstate((KA_T)in6p.in6p_ppcb, &t, fam)) == 1)
+			    return;
+		    }
+		}
 		enter_dev_ch(print_kptr((KA_T)(in6p.in6p_ppcb ? in6p.in6p_ppcb
 							      : s.so_pcb),
 					       (char *)NULL, 0));
-		if (p.pr_protocol == IPPROTO_TCP)
-		    ta = (KA_T)in6p.in6p_ppcb;
 	        la = (unsigned char *)&in6p.in6p_laddr;
 	        lp = (int)ntohs(in6p.in6p_lport);
 		if (!IN6_IS_ADDR_UNSPECIFIED(&in6p.in6p_faddr)
@@ -252,11 +349,15 @@ process_socket(sa)
 		    enter_nm(Namech);
 		    return;
 		}
+		if (p.pr_protocol == IPPROTO_TCP) {
+		    if (inp.inp_ppcb) {
+			if ((ts = ckstate((KA_T)inp.inp_ppcb, &t, fam)) == 1)
+			    return;
+		    }
+		}
 		enter_dev_ch(print_kptr((KA_T)(inp.inp_ppcb ? inp.inp_ppcb
 							    : s.so_pcb),
 					       (char *)NULL, 0));
-		if (p.pr_protocol == IPPROTO_TCP)
-		    ta = (KA_T)inp.inp_ppcb;
 		lp = (int)ntohs(inp.inp_lport);
 		if (fam == AF_INET) {
 
@@ -306,7 +407,7 @@ process_socket(sa)
  	 */
 	    if (fa || la)
 		(void) ent_inaddr(la, lp, fa, fp, fam);
-	    if (ta && !kread(ta, (char *)&t, sizeof(t))) {
+	    if (ts == 0) {
 		Lf->lts.type = 0;
 		Lf->lts.state.i = (int)t.t_state;
 
