@@ -31,27 +31,32 @@
 
 
 /*
- * $Id: machine.h,v 1.35 2005/08/29 10:21:45 abe Exp $
+ * $Id: machine.h,v 1.37 2008/10/21 16:16:06 abe Exp $
  */
 
 
 #if	!defined(LSOF_MACHINE_H)
 #define	LSOF_MACHINE_H	1
 
-
 #include <sys/types.h>
 
+# if	defined(HAS_CONF_MINOR)
+#undef	minor
+# endif	/* defined(HAS_CONF_MINOR) */
+
 #if	defined(HASCPUMASK_T)
 /*
- * In FreeBSD >= 5.2 when the cpumask_t typedef is present, it is defined int
- * <sys/types.h> only if _KERNEL is predefined.  However, predefining _KERNEL
- * before #include'ing <sys/types.h> causes redefinition errors for boolean_t
- * and vm_page_t when <vm/vm.h> is #include'd with _KERNEL predefined.  Since
- * lsof must have _KERNEL predefined when <vm/vm.h> is #include'd, the
- * expedient choice is made to duplicate the cpumask_t typedef here.
+ * In FreeBSD >= 5.2 when the cpumask_t typedef is present, it may be defined
+ * in <sys/types.h> only if _KERNEL is predefined.  However, predefining
+ * _KERNEL before #include'ing <sys/types.h> causes redefinition errors for
+ * boolean_t and vm_page_t when <vm/vm.h> is #include'd with _KERNEL
+ * predefined.  Since lsof must have _KERNEL predefined when <vm/vm.h> is
+ * #include'd, the expedient choice is made to duplicate the cpumask_t typedef
+ * here.
  *
  * Note: the Configure script defines HASCPUMASK_T if <sys/types.h> and
- * <machine/_types.h> both have cpumask_t references.
+ * <machine/_types.h> both have cpumask_t references and if the reference in
+ * <sys/types.h> is valid only when _KERNEL is defined.
  */
 
 typedef	__cpumask_t	cpumask_t;
@@ -163,10 +168,19 @@ typedef	__cpumask_t	cpumask_t;
  * FSV_DEFAULT defines the default set of file structure values to list.
  * It defaults to zero (0), but may be made up of a combination of the
  * FSV_* symbols from lsof.h.
+ *
+ *   HASNOFSADDR  -- has no file structure address
+ *   HASNOFSFLAGS -- has no file structure flags
+ *   HASNOFSCOUNT -- has no file structure count
+ *   HASNOFSNADDR -- has no file structure node address
  */
 
 #define	HASFSTRUCT	1
 /* #define	FSV_DEFAULT	FSV_? | FSV_? | FSV_? */
+/* #define	HASNOFSADDR	1	has no file structure address */
+/* #define	HASNOFSFLAGS	1	has no file structure flags */
+/* #define	HASNOFSCOUNT	1	has no file structure count */
+/* #define	HASNOFSNADDR	1	has no file structure node address */
 
 
 /*
@@ -516,6 +530,15 @@ typedef	__cpumask_t	cpumask_t;
 /* #define	HASTCPTPIW	1 */
 
 
+/*
+ * HASTCPUDPSTATE is defined for dialects that have TCP and UDP state
+ * support -- i.e., for the "-stcp|udp:state" option and its associated
+ * speed improvements.
+ */
+
+#define	HASTCPUDPSTATE	1
+
+
 /*
  * HASTMPNODE is defined for those dialects that have tmpnodes.
  */
