@@ -32,7 +32,7 @@
 #ifndef	lint
 static char copyright[] =
 "@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dmnt.c,v 1.16 2005/08/29 10:23:33 abe Exp $";
+static char *rcsid = "$Id: dmnt.c,v 1.17 2008/04/15 13:32:26 abe Exp $";
 #endif
 
 
@@ -189,6 +189,8 @@ getmntdev(dn, s, ss)
 	static int err = 0;
 	int h;
 	mntsup_t *mp, *mpn;
+	static char *vbuf = (char *)NULL;
+	static size_t vsz = (size_t)0;
 
 	if (err)
 	    return(0);
@@ -214,7 +216,7 @@ getmntdev(dn, s, ss)
 		err = 1;
 		return(0);
 	    }
-	    if (!(fs = fopen(MntSupP, "r"))) {
+	    if (!(fs = open_proc_stream(MntSupP, "r", &vbuf, &vsz, 0))) {
 
 	    /*
 	     * The mount supplement file can't be opened for reading.
@@ -423,17 +425,16 @@ readmnt()
 	FILE *ms;
 	int nfs;
 	struct stat sb;
+	static char *vbuf = (char *)NULL;
+	static size_t vsz = (size_t)0;
 
 	if (Lmi || Lmist)
 	    return(Lmi);
 /*
- * Open access to /proc/mounts
+ * Open access to /proc/mounts, assigning a page size buffer to its stream.
  */
 	(void) snpf(buf, sizeof(buf), "%s/mounts", PROCFS);
-	if (!(ms = fopen(buf, "r"))) {
-	    (void) fprintf(stderr, "%s: can't fopen(%s)\n", Pn, buf);
-	    Exit(1);
-	}
+	ms = open_proc_stream(buf, "r", &vbuf, &vsz, 1);
 /*
  * Read mount table entries.
  */
