@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.18 2006/03/27 22:57:11 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.20 2009/03/25 19:22:39 abe Exp $";
 #endif
 
 
@@ -109,8 +109,9 @@ get_fields(ln, sep, fr, eb, en)
 	char *ln;			/* input line */
 	char *sep;			/* separator list */
 	char ***fr;			/* field pointer return address */
-	int *eb;			/* indexes of fields where blank may be
-					 * embedded and is not a separator
+	int *eb;			/* indexes of fields where blank or an
+					 * entry from the separator list may be
+					 * embedded and are not separators
 					 * (may be NULL) */
 	int en;				/* number of entries in eb[] (may be
 					 * zero) */
@@ -131,9 +132,13 @@ get_fields(ln, sep, fr, eb, en)
 		    *cp = '\0';
 		    break;
 		}
-		if (*cp == '\t')
+		if (*cp == '\t')	/* TAB is always a separator */
 		    break;
 		if (*cp == ' ')  {
+
+		/*
+		 * See if this field may have an embedded space.
+		 */
 		    if (!eb || !en)
 			break;
 		    else {
@@ -148,12 +153,32 @@ get_fields(ln, sep, fr, eb, en)
 		    }
 		}
 		if (sep) {
+
+		/*
+		 * See if the character is in the separator list.
+		 */
 		    for (sp = sep; *sp; sp++) {
 			if (*sp == *cp)
 			    break;
 		    }
-		    if (*sp)
-			break;
+		    if (*sp) {
+
+		    /*
+		     * See if this field may have an embedded separator.
+		     */
+			if (!eb || !en)
+			    break;
+			else {
+			    for (i = j = 0; i < en; i++) {
+				if (eb[i] == n) {
+				    j = 1;
+				    break;
+				}
+			    }
+			    if (!j)
+				break;
+			}
+		    }
 		}
 	    }
 	    if (*cp)
@@ -191,11 +216,13 @@ get_locks(p)
 	char buf[MAXPATHLEN], *ec, **fp;
 	dev_t dev;
 	int ex, i, h, mode, pid;
-	unsigned long inode;
+	INODETYPE inode;
 	struct llock *lp, *np;
 	FILE *ls;
 	long maj, min;
 	char type;
+	static char *vbuf = (char *)NULL;
+	static size_t vsz = (size_t)0;
 /*
  * Destroy previous lock information.
  */
@@ -222,9 +249,10 @@ get_locks(p)
 	    }
 	}
 /*
- * Open /proc lock file and read it.
+ * Open the /proc lock file, assign a page size buffer to its stream,
+ * and read it.
  */
-	if (!(ls = fopen(p, "r")))
+	if (!(ls = open_proc_stream(p, "r", &vbuf, &vsz, 0)))
 	    return;
 	while (fgets(buf, sizeof(buf), ls)) {
 	    if (get_fields(buf, ":", &fp, (int *)NULL, 0) < 10)
@@ -267,7 +295,7 @@ get_locks(p)
 	 */
 	    ec = (char *)NULL;
 	    if (!fp[7] || !*fp[7]
-	    ||  (inode = strtoul(fp[7], &ec, 0)) == ULONG_MAX
+	    ||  (inode = strtoull(fp[7], &ec, 0)) == ULONG_MAX
 	    ||  !ec || *ec)
 		continue;
 	/*
@@ -297,7 +325,7 @@ get_locks(p)
 	    for (lp = LckH[h]; lp; lp = lp->next) {
 		if (lp->pid == pid
 		&&  lp->dev == dev
-		&&  lp->inode == (INODETYPE)inode
+		&&  lp->inode == inode
 		&&  lp->type == type)
 		    break;
 	    }
@@ -307,14 +335,15 @@ get_locks(p)
 	 * Allocate a new llock structure and link it to the PID hash bucket.
 	 */
 	    if (!(lp = (struct llock *)malloc(sizeof(struct llock)))) {
+		(void) snpf(buf, sizeof(buf), InodeFmt_d, inode);
 		(void) fprintf(stderr,
-		    "%s: can't allocate llock: PID %d; dev %x; inode %ld\n",
-		    Pn, pid, (int)dev, inode);
+		    "%s: can't allocate llock: PID %d; dev %x; inode %s\n",
+		    Pn, pid, (int)dev, buf);
 		Exit(1);
 	    }
 	    lp->pid = pid;
 	    lp->dev = dev;
-	    lp->inode = (INODETYPE)inode;
+	    lp->inode = inode;
 	    lp->type = type;
 	    lp->next = LckH[h];
 	    LckH[h] = lp;
@@ -336,7 +365,8 @@ process_proc_node(p, s, ss, l, ls)
 					 * others) */
 	int ls;				/* *l status -- i.e., SB_* values */
 {
-	mode_t access, type;
+	mode_t access;
+	mode_t type = 0;
 	char *cp;
 	struct mounts *mp = (struct mounts *)NULL;
 	size_t sz;
@@ -416,14 +446,14 @@ process_proc_node(p, s, ss, l, ls)
 	case N_BLK:
 	case N_CHR:
 	case N_FIFO:
-	    if (!Fsize && l && (ls & SB_SIZE)) {
+	    if (!Fsize && l && (ls & SB_SIZE) && OffType) {
 		Lf->off = (SZOFFTYPE)l->st_size;
 		Lf->off_def = 1;
 	    }
 	    break;
 	default:
 	    if (Foffset) {
-		if (l && (ls & SB_SIZE)) {
+		if (l && (ls & SB_SIZE) && OffType) {
 		    Lf->off = (SZOFFTYPE)l->st_size;
 		    Lf->off_def = 1;
 		}
@@ -466,6 +496,9 @@ process_proc_node(p, s, ss, l, ls)
 	    case S_IFLNK:
 		tn = "LINK";
 		break;
+	    case S_ISVTX:
+		tn = "VTXT";
+		break;
 	    default:
 		(void) snpf(Lf->type, sizeof(Lf->type), "%04o",
 		    ((type >> 12) & 0xf));
