@@ -32,42 +32,56 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.20 2007/04/24 16:20:53 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.22 2008/10/21 16:17:21 abe Exp $";
 #endif
 
 #include "lsof.h"
- 
-#if	defined(HASSELINUX)
-#include <selinux/selinux.h>
-#endif	/* defined(HASSELINUX) */
 
 
 /*
  * Local definitions
  */
 
+#define	FDINFO_FLAGS		1	/* fdinfo flags available */
+#define	FDINFO_POS		2	/* fdinfo position available */
+#define FDINFO_ALL		(FDINFO_FLAGS | FDINFO_POS)
 #define	LSTAT_TEST_FILE		"/"
 #define LSTAT_TEST_SEEK		1
 
+#if	!defined(ULLONG_MAX)
+#define	ULLONG_MAX		18446744073709551615ULL
+#endif	/* !defined(ULLONG_MAX) */
+
+
+/*
+ * Local structures
+ */
+
+struct l_fdinfo {
+	int flags;			/* flags: line value */
+	off_t pos;			/* pos: line value */
+};
+
 
 /*
  * Local variables
  */
 
-#if	defined(HASSELINUX)
-static cntxlist_t **CntxNm = (cntxlist_t **)NULL;
-
-#endif	/* defined(HASSELINUX) */
-
-short Cscko, Scko;			/* socket processing options */
-struct lproc *PLp;			/* process' Lp */
+static short Cckreg;			/* conditional status of regular file
+					 * checking:
+					 *     0 = unconditionally check
+					 *     1 = conditionally check */
+static short Ckscko;			/* socket file only checking status:
+					 *     0 = none
+					 *     1 = check only socket files */
 
 
 /*
  * Local function prototypes
  */
 
-_PROTOTYPE(static int getlinksrc,(char *ln, char *src, int srcl, short *slash));
+_PROTOTYPE(static int get_fdinfo,(char *p, struct l_fdinfo *fi));
+_PROTOTYPE(static int getlinksrc,(char *ln, char *src, int srcl));
 _PROTOTYPE(static int nm2id,(char *nm, int *id, int *idl));
 _PROTOTYPE(static int read_id_stat,(int ty, char *p, int id, char **cmd,
 				    int *ppid, int *pgid));
@@ -154,7 +168,7 @@ gather_proc_info()
 	static MALLOC_S pidx = 0;
 	static DIR *ps = (DIR *)NULL;
 	static char *taskpath = (char *)NULL;
-	static MALLOC_S taskpathl = 0;
+	static int taskpathl = 0;
 	DIR *ts;
 	UID_ARG uid;
 
@@ -196,32 +210,33 @@ gather_proc_info()
 
 	    /*
 	     * Selection ANDing or only network file options have been
-	     * specified, so set unconditional skipping of regular files.
+	     * specified, so set unconditional skipping of regular files
+	     * and socket file only checking.
 	     */
-		Cscko = 0;
-		Scko = 1;
+		Cckreg = 0;
+		Ckscko = 1;
 	    } else {
 
 	    /*
 	     * If ORed file selection options have been specified, or no ORed
-	     * process options have been specified, enable unconditional file
-	     * processing.
+	     * process selection options have been specified, enable
+	     * unconditional file checking and clear socket file only checking.
 	     *
 	     * If only ORed process selection options have been specified,
-	     * enable conditional file skipping.
+	     * enable conditional file skipping and socket file only checking.
 	     */
 		if ((Selflags & SELFILE) || !(Selflags & SELPROC))
-		    Cscko = Scko = 0;
+		    Cckreg = Ckscko = 0;
 		else
-		    Cscko = Scko = 1;
+		    Cckreg = Ckscko = 1;
 	    }
 	} else {
 
 	/*
 	 * No network file selection options were specified.  Enable
-	 * unconditional file processing.
+	 * unconditional file checking and clear socket file only checking.
 	 */
-	    Cscko = Scko = 0;
+	    Cckreg = Ckscko = 0;
 	}
 /*
  * Read /proc, looking for PID directories.  Open each one and
@@ -329,31 +344,91 @@ gather_proc_info()
 }
 
 
+/*
+ * get_fdinfo() - get values from /proc/<PID>fdinfo/FD
+ */
+
+static int
+get_fdinfo(p, fi)
+	char *p;			/* path to fdinfo file */
+	struct l_fdinfo *fi;		/* pointer to local fdinfo values
+					 * return structure */
+{
+	char buf[MAXPATHLEN + 1], *ep, **fp;
+	FILE *fs;
+	int rv = 0;
+	unsigned long ul;
+	unsigned long long ull;
+/*
+ * Signal no values returned (0) if no fdinfo pointer was provided or if the
+ * fdinfo path can't be opened.
+ */
+	if (!fi)
+	    return(0);
+	if (!p || !*p || !(fs = fopen(p, "r")))
+	    return(0);
+/*
+ * Read the fdinfo file.
+ */
+	while (fgets(buf, sizeof(buf), fs)) {
+	    if (get_fields(buf, (char *)NULL, &fp, (int *)NULL, 0) < 2)
+		continue;
+	    if (!fp[0] || !*fp[0] || !fp[1] || !*fp[1])
+		continue;
+	    if (!strcmp(fp[0], "flags:")) {
+
+	    /*
+	     * Process a "flags:" line.
+	     */
+		ep = (char *)NULL;
+		if ((ul = strtoul(fp[1], &ep, 0)) == ULONG_MAX
+		||  !ep || *ep)
+		    continue;
+		fi->flags = (unsigned int)ul;
+		if ((rv |= FDINFO_FLAGS) == FDINFO_ALL)
+		    break;
+	    } else if (!strcmp(fp[0], "pos:")) {
+
+	    /*
+	     * Process a "pos:" line.
+	     */
+		ep = (char *)NULL;
+		if ((ull = strtoull(fp[1], &ep, 0)) == ULLONG_MAX
+		||  !ep || *ep)
+		    continue;
+		fi->pos = (off_t)ull;
+		if ((rv |= FDINFO_POS) == FDINFO_ALL)
+		    break;
+	    }
+	}
+	fclose(fs);
+/*
+ * Signal via the return value what information was obtained. (0 == none)
+ */
+	return(rv);
+}
+
+
 /*
  * getlinksrc() - get the source path name for the /proc/<PID>/fd/<FD> link
  */
 
 
 static int
-getlinksrc(ln, src, srcl, slash)
+getlinksrc(ln, src, srcl)
 	char *ln;			/* link path */
 	char *src;			/* link source path return address */
 	int srcl;			/* length of src[] */
-	short *slash;			/* leading slash flag: 0 = not present
-					 *		       1 = present */
 {
 	char *cp;
 	int ll;
 
-	*slash = 0;
 	if ((ll = readlink(ln, src, srcl - 1)) < 1
 	||  ll >= srcl)
 	    return(-1);
 	src[ll] = '\0';
-	if (*src == '/') {
-	    *slash = 1;
+	if (*src == '/')
 	    return(ll);
-	}
 	if ((cp = strchr(src, ':'))) {
 	    *cp = '\0';
 	    ll = strlen(src);
@@ -370,7 +445,7 @@ void
 initialize()
 {
 	int fd;
-	int off = 0;
+	struct l_fdinfo fi;
 	char path[MAXPATHLEN];
 	struct stat sb;
 /*
@@ -384,7 +459,11 @@ initialize()
 /*
  * Open LSTAT_TEST_FILE and seek to byte LSTAT_TEST_SEEK, then lstat the
  * /proc/<PID>/fd/<FD> for LSTAT_TEST_FILE to see what position is reported.
- * If the result isn't LSTAT_TEST_SEEK, disable offset * reporting.
+ * If the result is LSTAT_TEST_SEEK, enable offset reporting.
+ *
+ * If the result isn't LSTAT_TEST_SEEK, next check the fdinfo file for the
+ * open LSTAT_TEST_FILE file descriptor.  If it exists and contains a "pos:"
+ * value, and if the value is LSTAT_TEST_SEEK, enable offset reporting.
  */
 	if ((fd = open(LSTAT_TEST_FILE, O_RDONLY)) >= 0) {
 	    if (lseek(fd, (off_t)LSTAT_TEST_SEEK, SEEK_SET)
@@ -393,12 +472,20 @@ initialize()
 			    fd);
 		if (!lstat(path, &sb)) {
 		    if (sb.st_size == (off_t)LSTAT_TEST_SEEK)
-			off = 1;
+			OffType = 1;
+		}
+	    }
+	    if (!OffType) {
+		(void) snpf(path, sizeof(path), "%s/%d/fdinfo/%d", PROCFS,
+			    Mypid, fd);
+		if (get_fdinfo(path, &fi) & FDINFO_POS) {
+		    if (fi.pos == (off_t)LSTAT_TEST_SEEK)
+			OffType = 2;
 		}
 	    }
 	    (void) close(fd);
 	}
-	if (!off) {
+	if (!OffType) {
 	    if (Foffset && !Fwarn)
 		(void) fprintf(stderr,
 		    "%s: WARNING: can't report offset; disregarding -o.\n",
@@ -406,6 +493,13 @@ initialize()
 	    Foffset = 0;
 	    Fsize = 1;
 	}
+	if (Fsv && (OffType != 2)) {
+	    if (!Fwarn && FsvByf)
+		(void) fprintf(stderr,
+		    "%s: WARNING: can't report file flags; disregarding +f.\n",
+		    Pn);
+	    Fsv = 0;
+	}
 /*
  * Make sure the local mount info table is loaded if doing anything other
  * than just Internet lookups.  (HasNFS is defined during the loading of the
@@ -495,6 +589,74 @@ nm2id(nm, id, idl)
 }
 
 
+/*
+ * open_proc_stream() -- open a /proc stream
+ */
+
+FILE *
+open_proc_stream(p, m, buf, sz, act)
+	char *p;			/* pointer to path to open */
+	char *m;			/* pointer to mode -- e.g., "r" */
+	char **buf;			/* pointer tp setvbuf() address
+					 * (NULL if none) */
+	size_t *sz;			/* setvbuf() size (0 if none or if
+					 * getpagesize() desired */
+	int act;			/* fopen() failure action:
+					 *     0 : return (FILE *)NULL
+					 *   <>0 : fprintf() an error message
+					 *         and Exit(1)
+					 */
+{
+	FILE *fs;			/* opened stream */
+	static size_t psz = (size_t)0;	/* page size */
+	size_t tsz;			/* temporary size */
+/*
+ * Open the stream.
+ */
+	if (!(fs = fopen(p, m))) {
+	    if (!act)
+		return((FILE *)NULL);
+	    (void) fprintf(stderr, "%s: can't fopen(%s, \"%s\"): %s\n",
+		Pn, p, m, strerror(errno));
+	    Exit(1);
+	}
+/*
+ * Return the stream if no buffer change is required.
+ */
+	if (!buf)
+	    return(fs);
+/*
+ * Determine the buffer size required.
+ */
+	if (!(tsz = *sz)) {
+	    if (!psz)
+		psz = getpagesize();
+	    tsz = psz;
+	}
+/*
+ * Allocate a buffer for the stream, as required.
+ */
+	if (!*buf) {
+	    if (!(*buf = (char *)malloc((MALLOC_S)tsz))) {
+		(void) fprintf(stderr,
+		    "%s: can't allocate %d bytes for %s stream buffer\n",
+		    Pn, (int)tsz, p);
+		Exit(1);
+	    }
+	    *sz = tsz;
+	}
+/*
+ * Assign the buffer to the stream.
+ */
+	if (setvbuf(fs, *buf, _IOFBF, tsz)) {
+	    (void) fprintf(stderr, "%s: setvbuf(%s)=%d failure: %s\n",
+		Pn, p, (int)tsz, strerror(errno));
+	    Exit(1);
+	}
+	return(fs);
+}
+
+
 /*
  * process_id - process ID: PID or LWP
  *
@@ -512,16 +674,23 @@ process_id(idp, idpl, cmd, uid, pid, ppid, pgid)
 	int ppid;			/* parent PID */
 	int pgid;			/* parent GID */
 {
+	int av;
 	static char *dpath = (char *)NULL;
 	static int dpathl = 0;
-	short enls, enss, lnk, pn, pss, sf, slash;
+	short enls, enss, lnk, oty, pn, pss, sf;
 	int fd, i, ls, n, ss, sv;
+	struct l_fdinfo fi;
 	DIR *fdp;
 	struct dirent *fp;
+	static char *ipath = (char *)NULL;
+	static int ipathl = 0;
+	int j = 0;
 	struct stat lsb, sb;
 	char nmabuf[MAXPATHLEN + 1], pbuf[MAXPATHLEN + 1];
 	static char *path = (char *)NULL;
 	static int pathl = 0;
+	static char *pathi = (char *)NULL;
+	static int pathil = 0;
 	int txts = 0;
 
 #if	defined(HASSELINUX)
@@ -534,36 +703,36 @@ process_id(idp, idpl, cmd, uid, pid, ppid, pgid)
 	if (is_proc_excl(pid, pgid, uid, &pss, &sf)
 	||  is_cmd_excl(cmd, &pss, &sf))
 	    return(1);
-/*
- * Set the socket processing options.
- */
-	if (Cscko) {
-	    if (sf & SELPROC)
-		Scko = 0;
-		Scko = 1;
+	if (Cckreg) {
+
+	/*
+	 * If conditional checking of regular files is enabled, enable
+	 * socket file only checking, based on the process' selection
+	 * status.
+	 */
+	    Ckscko = (sf & SELPROC) ? 0 : 1;
 	}
 	alloc_lproc(pid, pgid, ppid, uid, cmd, (int)pss, (int)sf);
 	Plf = (struct lfile *)NULL;
 /*
  * Process the ID's current working directory info.
  */
-	(void) make_proc_path(idp, idpl, &path, &pathl, "cwd");
-	alloc_lfile(CWD, -1);
-	if (getlinksrc(path, pbuf, sizeof(pbuf), &slash) < 1) {
-	    if (!Fwarn) {
-		(void) memset((void *)&sb, 0, sizeof(sb));
-		lnk = ss = 0;
-		(void) snpf(nmabuf, sizeof(nmabuf), "(readlink: %s)",
-		    strerror(errno));
-		nmabuf[sizeof(nmabuf) - 1] = '\0';
-		(void) add_nma(nmabuf, strlen(nmabuf));
-		pn = 1;
-	    } else
-		pn = 0;
-	} else {
-	    lnk = 1;
-	    if ((Scko && !slash) || !Scko) {
-		pn = 1;
+	if (!Ckscko) {
+	    (void) make_proc_path(idp, idpl, &path, &pathl, "cwd");
+	    alloc_lfile(CWD, -1);
+	    if (getlinksrc(path, pbuf, sizeof(pbuf)) < 1) {
+		if (!Fwarn) {
+		    (void) memset((void *)&sb, 0, sizeof(sb));
+		    lnk = ss = 0;
+		    (void) snpf(nmabuf, sizeof(nmabuf), "(readlink: %s)",
+			strerror(errno));
+		    nmabuf[sizeof(nmabuf) - 1] = '\0';
+		    (void) add_nma(nmabuf, strlen(nmabuf));
+		    pn = 1;
+		} else
+		    pn = 0;
+	    } else {
+		lnk = pn = 1;
 		ss = SB_ALL;
 		if (HasNFS) {
 		    if ((sv = statsafely(path, &sb)))
@@ -579,36 +748,35 @@ process_id(idp, idpl, cmd, uid, pid, ppid, pgid)
 			(void) add_nma(nmabuf, strlen(nmabuf));
 		    }
 		}
-	    } else
-		pn = 0;
-	}
-	if (pn) {
-	    (void) process_proc_node(lnk ? pbuf : path,
-				     &sb, ss,
-				     (struct stat *)NULL, 0);
-	    if (Lf->sf)
-		link_lfile();
+	    }
+	    if (pn) {
+		(void) process_proc_node(lnk ? pbuf : path,
+					 &sb, ss,
+					 (struct stat *)NULL, 0);
+		if (Lf->sf)
+		    link_lfile();
+	    }
 	}
 /*
  * Process the ID's root directory info.
  */
-	(void) make_proc_path(idp, idpl, &path, &pathl, "root");
-	alloc_lfile(RTD, -1);
-	if (getlinksrc(path, pbuf, sizeof(pbuf), &slash) < 1) {
-	    if (!Fwarn) {
-		(void) memset((void *)&sb, 0, sizeof(sb));
-		lnk = ss = 0;
-		(void) snpf(nmabuf, sizeof(nmabuf), "(readlink: %s)",
-		    strerror(errno));
-		nmabuf[sizeof(nmabuf) - 1] = '\0';
-		(void) add_nma(nmabuf, strlen(nmabuf));
-		pn = 1;
-	    } else
-		pn = 0;
-	} else {
-	    lnk = 1;
-	    if ((Scko && !slash) || !Scko) {
-		pn = 1;
+	if (!Ckscko) {
+	    (void) make_proc_path(idp, idpl, &path, &pathl, "root");
+	    alloc_lfile(RTD, -1);
+	    if (getlinksrc(path, pbuf, sizeof(pbuf)) < 1) {
+		if (!Fwarn) {
+		    (void) memset((void *)&sb, 0, sizeof(sb));
+		    lnk = ss = 0;
+		    (void) snpf(nmabuf, sizeof(nmabuf), "(readlink: %s)",
+			strerror(errno));
+		    nmabuf[sizeof(nmabuf) - 1] = '\0';
+		    (void) add_nma(nmabuf, strlen(nmabuf));
+		    pn = 1;
+		} else
+		    pn = 0;
+	    } else {
+		lnk = pn = 1;
+		ss = SB_ALL;
 		if (HasNFS) {
 		    if ((sv = statsafely(path, &sb)))
 			sv = statEx(pbuf, &sb, &ss);
@@ -622,40 +790,38 @@ process_id(idp, idpl, cmd, uid, pid, ppid, pgid)
 			nmabuf[sizeof(nmabuf) - 1] = '\0';
 			(void) add_nma(nmabuf, strlen(nmabuf));
 		    }
-		} else
-		    ss = SB_ALL;
-	    } else
-		pn = 0;
-	}
-	if (pn) {
-	    (void) process_proc_node(lnk ? pbuf : path,
-				     &sb, ss,
-				     (struct stat *)NULL, 0);
-	    if (Lf->sf)
-		link_lfile();
+		}
+	    }
+	    if (pn) {
+		(void) process_proc_node(lnk ? pbuf : path,
+					 &sb, ss,
+					 (struct stat *)NULL, 0);
+		if (Lf->sf)
+		    link_lfile();
+	    }
 	}
 /*
  * Process the ID's execution info.
  */
-	(void) make_proc_path(idp, idpl, &path, &pathl, "exe");
-	alloc_lfile("txt", -1);
-	if (getlinksrc(path, pbuf, sizeof(pbuf), &slash) < 1) {
-	    (void) memset((void *)&sb, 0, sizeof(sb));
-	    lnk = ss = 0;
-	    if (!Fwarn) {
-		if ((errno != ENOENT) || uid) {
-		    (void) snpf(nmabuf, sizeof(nmabuf), "(readlink: %s)",
-			strerror(errno));
-		    nmabuf[sizeof(nmabuf) - 1] = '\0';
-		    (void) add_nma(nmabuf, strlen(nmabuf));
-		}
-		pn = 1;
-	    } else
-		pn = 0;
-	 } else {
-	    lnk = 1;
-	    if ((Scko && !slash) || !Scko) {
-		pn = 1;
+	if (!Ckscko) {
+	    (void) make_proc_path(idp, idpl, &path, &pathl, "exe");
+	    alloc_lfile("txt", -1);
+	    if (getlinksrc(path, pbuf, sizeof(pbuf)) < 1) {
+		(void) memset((void *)&sb, 0, sizeof(sb));
+		lnk = ss = 0;
+		if (!Fwarn) {
+		    if ((errno != ENOENT) || uid) {
+			(void) snpf(nmabuf, sizeof(nmabuf), "(readlink: %s)",
+			    strerror(errno));
+			nmabuf[sizeof(nmabuf) - 1] = '\0';
+			(void) add_nma(nmabuf, strlen(nmabuf));
+		    }
+		    pn = 1;
+		} else
+		    pn = 0;
+	    } else {
+		lnk = pn = 1;
+		ss = SB_ALL;
 		if (HasNFS) {
 		    if ((sv = statsafely(path, &sb))) {
 			sv = statEx(pbuf, &sb,  &ss);
@@ -663,7 +829,7 @@ process_id(idp, idpl, cmd, uid, pid, ppid, pgid)
 			    txts = 1;
 		    }
 		} else
-			sv = stat(path, &sb);
+		    sv = stat(path, &sb);
 		if (sv) {
 		    ss = 0;
 		    if (!Fwarn) {
@@ -672,24 +838,21 @@ process_id(idp, idpl, cmd, uid, pid, ppid, pgid)
 			nmabuf[sizeof(nmabuf) - 1] = '\0';
 			(void) add_nma(nmabuf, strlen(nmabuf));
 		    }
-		} else {
-		    ss = SB_ALL;
+		} else
 		    txts = 1;
-		}
-	    } else
-		pn = 0;
-	}
-	if (pn) {
+	    }
+	    if (pn) {
 		(void) process_proc_node(lnk ? pbuf : path,
 					 &sb, ss,
 					 (struct stat *)NULL, 0);
 		if (Lf->sf)
 		    link_lfile();
+	    }
 	}
 /*
  * Process the ID's memory map info.
  */
-	if (!Scko) {
+	if (!Ckscko) {
 	    (void) make_proc_path(idp, idpl, &path, &pathl, "maps");
 	    (void) process_proc_map(path, txts ? &sb : (struct stat *)NULL,
 				    txts ? ss : 0);
@@ -709,7 +872,7 @@ process_id(idp, idpl, cmd, uid, pid, ppid, pgid)
 		Lp->cntx = (char *)NULL;
 		if (!Fwarn) {
 		    (void) snpf(nmabuf, sizeof(nmabuf),
-			"(getpidcon: %s)\n", strerror(errno));
+			"(getpidcon: %s)", strerror(errno));
 		    if (!(Lp->cntx = strdup(nmabuf))) {
 			(void) fprintf(stderr,
 			    "%s: no context error space: PID %ld",
@@ -740,6 +903,11 @@ process_id(idp, idpl, cmd, uid, pid, ppid, pgid)
 	if ((i = make_proc_path(idp, idpl, &dpath, &dpathl, "fd/")) < 3)
 	    return(0);
 	dpath[i - 1] = '\0';
+	if ((OffType == 2)
+	&&  ((j = make_proc_path(idp, idpl, &ipath, &ipathl, "fdinfo/")) >= 7))
+	    oty = 1;
+	else
+	    oty = 0;
 	if (!(fdp = opendir(dpath))) {
 	    if (!Fwarn) {
 		(void) snpf(nmabuf, sizeof(nmabuf), "%s (opendir: %s)",
@@ -757,7 +925,7 @@ process_id(idp, idpl, cmd, uid, pid, ppid, pgid)
 		continue;
 	    (void) make_proc_path(dpath, i, &path, &pathl, fp->d_name);
 	    (void) alloc_lfile((char *)NULL, fd);
-	    if (getlinksrc(path, pbuf, sizeof(pbuf), &slash) < 1) {
+	    if (getlinksrc(path, pbuf, sizeof(pbuf)) < 1) {
 		(void) memset((void *)&sb, 0, sizeof(sb));
 		lnk = ss = 0;
 		if (!Fwarn) {
@@ -770,41 +938,65 @@ process_id(idp, idpl, cmd, uid, pid, ppid, pgid)
 		    pn = 0;
 	    } else {
 		lnk = 1;
-		if ((Scko && !slash) || !Scko) {
-		    pn = 1;
-		    if (HasNFS) {
-			if (lstatsafely(path, &lsb)) {
-			    (void) statEx(pbuf, &lsb, &ls);
-			    enls = errno;
-			} else
-			ls = SB_ALL;
-			if (statsafely(path, &sb)) {
-			    (void) statEx(pbuf, &sb, &ss);
-			    enss = errno;
-			} else
-			    ss = SB_ALL;
+		if (HasNFS) {
+		    if (lstatsafely(path, &lsb)) {
+			(void) statEx(pbuf, &lsb, &ls);
+		        enls = errno;
 		    } else {
-			ls = lstat(path, &lsb) ? 0 : SB_ALL;
-			enls = errno;
-			ss = stat(path, &sb) ? 0 : SB_ALL;
+			enls = 0;
+			ls = SB_ALL;
+		    }
+		    if (statsafely(path, &sb)) {
+			(void) statEx(pbuf, &sb, &ss);
 			enss = errno;
+		    } else {
+			enss = 0;
+			ss = SB_ALL;
 		    }
-		    if (!ls && !Fwarn) {
-			(void) snpf(nmabuf, sizeof(nmabuf), "lstat: %s)",
-			    strerror(enls));
-			nmabuf[sizeof(nmabuf) - 1] = '\0';
-			(void) add_nma(nmabuf, strlen(nmabuf));
-		    }
-		    if (!ss && !Fwarn) {
-			(void) snpf(nmabuf, sizeof(nmabuf), "(stat: %s)",
-			    strerror(enss));
-			nmabuf[sizeof(nmabuf) - 1] = '\0';
-			(void) add_nma(nmabuf, strlen(nmabuf));
-		    }
+		} else {
+		    ls = lstat(path, &lsb) ? 0 : SB_ALL;
+		    enls = errno;
+		    ss = stat(path, &sb) ? 0 : SB_ALL;
+		    enss = errno;
+		}
+		if (!ls && !Fwarn) {
+		    (void) snpf(nmabuf, sizeof(nmabuf), "lstat: %s)",
+			strerror(enls));
+		    nmabuf[sizeof(nmabuf) - 1] = '\0';
+		    (void) add_nma(nmabuf, strlen(nmabuf));
+		}
+		if (!ss && !Fwarn) {
+		    (void) snpf(nmabuf, sizeof(nmabuf), "(stat: %s)",
+			strerror(enss));
+		    nmabuf[sizeof(nmabuf) - 1] = '\0';
+		    (void) add_nma(nmabuf, strlen(nmabuf));
+		}
+		if (Ckscko) {
+		    if ((ss & SB_MODE) && ((sb.st_mode & S_IFMT) == S_IFSOCK))
+			pn = 1;
+		    else
+			pn = 0;
 		} else
-		    pn = 0;
+		    pn = 1;
 	    }
 	    if (pn) {
+		if (oty) {
+		    (void) make_proc_path(ipath, j, &pathi, &pathil,
+					  fp->d_name);
+		    if ((av = get_fdinfo(pathi, &fi)) & FDINFO_POS) {
+			ls |= SB_SIZE;
+			lsb.st_size = fi.pos;
+		    } else
+			ls &= ~SB_SIZE;
+
+#if	!defined(HASNOFSFLAGS)
+		    if ((av & FDINFO_FLAGS) && (Fsv & FSV_FG)) {
+			Lf->ffg = (long)fi.flags;
+			Lf->fsv |= FSV_FG;
+		     }
+# endif	/* !defined(HASNOFSFLAGS) */
+
+		}
 		process_proc_node(lnk ? pbuf : path,
 				  &sb, ss, &lsb, ls);
 		if (Lf->sf)
@@ -842,10 +1034,13 @@ process_proc_map(p, s, ss)
 	};
 	static struct saved_map *sm = (struct saved_map *)NULL;
 	static int sma = 0;
+	static char *vbuf = (char *)NULL;
+	static size_t vsz = (size_t)0;
 /*
- * Open and read the /proc/<pid>/maps file.
+ * Open the /proc/<pid>/maps file, assign a page size buffer to its stream,
+ * and read it/
  */
-	if (!(ms = fopen(p, "r")))
+	if (!(ms = open_proc_stream(p, "r", &vbuf, &vsz, 0)))
 	    return;
 	while (fgets(buf, sizeof(buf), ms)) {
 	    if ((nf = get_fields(buf, ":", &fp, &eb, 1)) < 7)
@@ -882,7 +1077,10 @@ process_proc_map(p, s, ss)
 	    dev = (dev_t)makedev((int)maj, (int)min);
 	    if (!fp[5] || !*fp[5])
 		continue;
-	    inode = (INODETYPE)atoi(fp[5]);
+	    ep = (char *)NULL;
+	    if ((inode = strtoull(fp[5], &ep, 0)) == ULLONG_MAX
+	    ||  !ep || *ep)
+		continue;
 	    if (!dev && !inode)
 		continue;
 	/*
@@ -1030,10 +1228,13 @@ read_id_stat(ty, p, id, cmd, ppid, pgid)
 	FILE *fs;
 	MALLOC_S len;
 	int nf;
+	static char *vbuf = (char *)NULL;
+	static size_t vsz = (size_t)0;
 /*
- * Open the stat file path and read its first line.
+ * Open the stat file path, assign a page size buffer to its stream,
+ * and read the file's first line.
  */
-	if (!(fs = fopen(p, "r")))
+	if (!(fs = open_proc_stream(p, "r", &vbuf, &vsz, 0)))
 	    return(1);
 	cp = fgets(buf, sizeof(buf), fs);
 	(void) fclose(fs);
