@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: machine.h,v 1.30 2007/04/24 16:20:53 abe Exp $
+ * $Id: machine.h,v 1.33 2008/10/21 16:17:21 abe Exp $
  */
 
 
@@ -161,12 +161,12 @@
  *   HASNOFSNADDR -- has no file structure node address
  */
 
-/* #define	HASFSTRUCT	1 */
+#define	HASFSTRUCT	1
 /* #define	FSV_DEFAULT	FSV_? | FSV_? | FSV_? */
-/* #define	HASNOFSADDR	1	has no file structure address */
+#define	HASNOFSADDR	1	/* has no file structure address */
 /* #define	HASNOFSFLAGS	1	has no file structure flags */
-/* #define	HASNOFSCOUNT	1	has no file structure count */
-/* #define	HASNOFSNADDR	1	has no file structure node address */
+#define	HASNOFSCOUNT	1	/* has no file structure count */
+#define	HASNOFSNADDR	1	/* has no file structure node address */
 
 
 /*
@@ -504,6 +504,15 @@
 /* #define	HASTCPTPIW	1 */
 
 
+/*
+ * HASTCPUDPSTATE is defined for dialects that have TCP and UDP state
+ * support -- i.e., for the "-stcp|udp:state" option and its associated
+ * speed improvements.
+ */
+
+#define	HASTCPUDPSTATE	1
+
+
 /*
  * HASTMPNODE is defined for those dialects that have tmpnodes.
  */
@@ -527,7 +536,7 @@
  * option's default binary value -- 0 or 1.
  */
 
-#define	HASXOPT		"skip TCP&UDP files"
+#define	HASXOPT		"skip TCP&UDP* files"
 #define	HASXOPT_VALUE	0
 
 
