@@ -35,7 +35,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode1.c,v 1.7 2004/07/06 19:20:18 abe Exp $";
+static char *rcsid = "$Id: dnode1.c,v 1.8 2005/08/08 19:53:24 abe Exp $";
 #endif
 
 
@@ -77,7 +77,7 @@ int
 read_iso_node(v, d, ino, nl, sz)
 	struct vnode *v;		/* containing vnode */
 	dev_t *d;			/* returned device number */
-	unsigned long *ino;		/* returned inode number */
+	INODETYPE *ino;			/* returned inode number */
 	long *nl;			/* returned link count */
 	SZOFFTYPE *sz;			/* returned size */
 {
@@ -87,7 +87,7 @@ read_iso_node(v, d, ino, nl, sz)
 	||  kread((KA_T)v->v_data, (char *)&i, sizeof(i)))
 	    return(1);
 	*d = i.i_dev;
-	*ino = i.i_number;
+	*ino = (INODETYPE)i.i_number;
 	*nl = i.inode.iso_links;
 	*sz = (SZOFFTYPE)i.i_size;
 	return(0);
