@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: machine.h,v 1.42 2007/04/24 16:23:15 abe Exp $
+ * $Id: machine.h,v 1.44 2009/03/25 19:22:16 abe Exp $
  */
 
 
@@ -57,6 +57,15 @@
  */
 
 typedef struct aio_req { int dummy; } aio_req_t;
+
+/*
+ * Include <sys/utsname.h> so it won't be corrupted for 32 bit compilations
+ * when _KERNEL is defined for some include files in dlsof.h.
+ *
+ * Daniel Trinkle identified this requirement.
+ */
+
+#include <sys/utsname.h>
 # endif	/* solaris>=100000 */
 
 
@@ -380,7 +389,10 @@ typedef struct aio_req { int dummy; } aio_req_t;
  * called from print_file().
  */
 
+# if	solaris<100000
 #define	HASPRINTDEV	print_dev
+# endif	/* solaris<100000 */
+
 /* #define	HASPRINTINO	print_ino?	*/
 /* #define	HASPRINTNM	print_nm?	*/
 /* #define	HASPRINTOFF	print_off?	*/
@@ -571,6 +583,15 @@ typedef struct aio_req { int dummy; } aio_req_t;
 # endif	/* solaris==20300 || solaris>=20500 */
 
 
+/*
+ * HASTCPUDPSTATE is defined for dialects that have TCP and UDP state
+ * support -- i.e., for the "-stcp|udp:state" option and its associated
+ * speed improvements.
+ */
+
+#define	HASTCPUDPSTATE	1
+
+
 /*
  * HASTMPNODE is defined for those dialects that have tmpnodes.
  */
