@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: lsof.h,v 1.62 2009/03/25 19:20:38 abe Exp $
+ * $Id: lsof.h,v 1.63 2010/01/18 19:01:11 abe Exp $
  */
 
 
@@ -267,11 +267,13 @@ extern int optind;
 # if	!defined(LOGINML)
 #  if	defined(HASUTMPX)
 static struct utmpx dummy_utmp;		/* to get login name length */
-# else	/* !defined(HASUTMPX) */
+#define	LOGINML		sizeof(dummy_utmp.ut_user)
+					/* login name length */
+#  else	/* !defined(HASUTMPX) */
 static struct utmp dummy_utmp;		/* to get login name length */
-# endif	/* defined(HASUTMPX) */
 #define	LOGINML		sizeof(dummy_utmp.ut_name)
 					/* login name length */
+#  endif	/* defined(HASUTMPX) */
 # endif	/* !defined(LOGINML) */
 
 #define	LPROCINCR	128		/* Lproc[] allocation increment */
@@ -336,15 +338,16 @@ static struct utmp dummy_utmp;		/* to get login name length */
 #define	N_PSEU		49		/* pseudofs node */
 #define	N_SAMFS		41		/* Solaris SAM-FS */
 #define	N_SANFS		42		/* AIX SANFS */
-#define	N_SHARED	43		/* Solaris sharedfs */
-#define	N_SOCK		44		/* sock_vnodeops node */
-#define	N_SPEC		45		/* spec_vnodeops node */
-#define	N_STREAM	46		/* stream node */
-#define	N_TMP		47		/* tmpfs node */
-#define	N_UFS		48		/* UNIX file system node */
-#define	N_VXFS		49		/* Veritas file system node */
-#define	N_XFS		50		/* XFS node */
-#define	N_ZFS		51		/* ZFS node */
+#define	N_SDEV		43		/* Solaris sdev file system node */
+#define	N_SHARED	44		/* Solaris sharedfs */
+#define	N_SOCK		45		/* sock_vnodeops node */
+#define	N_SPEC		46		/* spec_vnodeops node */
+#define	N_STREAM	47		/* stream node */
+#define	N_TMP		48		/* tmpfs node */
+#define	N_UFS		49		/* UNIX file system node */
+#define	N_VXFS		50		/* Veritas file system node */
+#define	N_XFS		51		/* XFS node */
+#define	N_ZFS		52		/* ZFS node */
 
 # if	!defined(OFFDECDIG)
 #define	OFFDECDIG	8		/* maximum number of digits in the
