@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: proc.c,v 1.43 2008/04/15 13:24:19 abe Exp $";
+static char *rcsid = "$Id: proc.c,v 1.45 2009/03/25 19:20:44 abe Exp $";
 #endif
 
 
@@ -566,6 +566,15 @@ is_cmd_excl(cmd, pss, sf)
 {
 	int i;
 	struct str_lst *sp;
+/*
+ * See if the command is excluded by a "-c^<command>" option.
+ */
+	if (Cmdl && Cmdnx) {
+	    for (sp = Cmdl; sp; sp = sp->next) {
+		if (sp->x && !strncmp(sp->str, cmd, sp->len))
+		    return(1);
+	    }
+	}
 /*
  * The command is not excluded if no command selection was requested,
  * or if its name matches any -c <command> specification.
@@ -574,7 +583,7 @@ is_cmd_excl(cmd, pss, sf)
 	if ((Selflags & SELCMD) == 0)
 	    return(0);
 	for (sp = Cmdl; sp; sp = sp->next) {
-	    if (strncmp(sp->str, cmd, sp->len) == 0) {
+	    if (!sp->x && !strncmp(sp->str, cmd, sp->len)) {
 		sp->f = 1;
 		*pss |= PS_PRI;
 		*sf |= SELCMD;
@@ -616,6 +625,8 @@ is_file_sel(lp, lf)
 {
 	if (!lf || !lf->sf)
 	    return(0);
+	if (Lf->sf & SELEXCLF)
+	    return(0);
 
 #if	defined(HASSECURITY) && defined(HASNOSOCKSECURITY)
 	if (Myuid && (Myuid != lp->uid)) {
@@ -828,6 +839,8 @@ is_proc_excl(pid, pgid, uid, pss, sf)
 void
 link_lfile()
 {
+	if (Lf->sf & SELEXCLF)
+	    return;
 	Lp->pss |= PS_SEC;
 	if (Plf)
 	    Plf->next = Lf;
